/*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*\

  System Core TypeScript Library

  Copyright (c) TypeScriptLibs and Contributors

  Licensed under the MIT License; you may not use this file except in
  compliance with the License. You may obtain a copy of the MIT License at
  https://typescriptlibs.org/LICENSE.txt

\*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*/
import ICollection from './ICollection';
import IDictionary from './IDictionary';
import IDictionaryEnumerator from './IDictionaryEnumerator';
import KeyValuePair from './KeyValuePair';
import Object from '../Object';
import Type from '../Type';
/**
 * This class provides as simple dictionary list for small numbers of items and
 * dictionary operations.
 *
 * @extends Object
 *
 * @implements IDictionary
 */
export declare class LinkedList<TKey extends Object, TValue extends Object> extends Object implements IDictionary<TKey, TValue> {
    /**
     * Initialize a new LinkedList.
     */
    constructor();
    private _count;
    private _endNode;
    private _startNode;
    private _version;
    /**
     * The number of items in the LinkedList.
     */
    readonly count: number;
    /**
     * Indicates whether this LinkedList is read-only.
     */
    readonly isReadOnly: boolean;
    /**
     * A collection of every key in the LinkedList.
     */
    readonly keys: ICollection<TKey>;
    /**
     * A collection of every value in the LinkedList.
     */
    readonly values: ICollection<TValue>;
    /**
     * Adds a new item to the LinkedList.
     *
     * @param {KeyValuePair} item
     * The item to add to the LinkedList.
     *
     * @exception {ArgumentNullException}
     * The key is null.
     *
     * @exception {ArgumentException}
     * An item with the same key already exists in the LinkedList.
     *
     * @exception {NotSupportedException}
     * The LinkedList is read-only.
     */
    add(item: KeyValuePair<TKey, TValue>): void;
    /**
     * Adds a specified item to the LinkedList.
     *
     * @param {TKey} key
     * The key of the item to add to the LinkedList.
     *
     * @param {TValue} value
     * The value of the item to add to the LinkedList.
     *
     * @exception {ArgumentNullException}
     * The key is null.
     *
     * @exception {ArgumentException}
     * An item with the same key already exists in the LinkedList.
     *
     * @exception {NotSupportedException}
     * The LinkedList is read-only.
     */
    addItem(key: TKey, value: TValue): void;
    /**
     * Removes all items from the ICollection.
     *
     * @exception {NotSupportedException}
     * The ICollection is read-only.
     */
    clear(): void;
    /**
     * Tests whether an item is part of the LinkedList.
     *
     * @param {KeyValuePair} item
     * The item to locate in the LinkedList.
     *
     * @return {boolean}
     * Returns true if the item has been found in the LinkedList, otherwise
     * returns false.
     */
    contains(item: KeyValuePair<TKey, TValue>): boolean;
    /**
     * Tests whether an item with the specified key is part of the LinkedList.
     *
     * @param {TKey} key
     * The key to locate in the LinkedList.
     *
     * @return {boolean}
     * Returns true if the LinkedList contains an item with the key, otherwise
     * false.
     *
     * @exception {ArgumentNullException}
     * The key is null.
     */
    containsKey(key: TKey): boolean;
    /**
     * Copies items of the ICollection to a specified array, starting at a
     * specified index of the array.
     *
     * @param {KeyValuePair[]} array
     * The array as destination of the items, that are copied from ICollection.
     *
     * @param {number} arrayIndex
     * The zero-based index of the array, at which to start with inserting the
     * copied items.
     *
     * @exception {ArgumentNullException}
     * The array is null.
     *
     * @exception {ArgumentOutOfRangeException}
     * The array index is less than zero.
     *
     * @exception {ArgumentException}
     * The number of items in the ICollection is greater than the available
     * space from arrayIndex to the end of the array.
     */
    copyTo(array: KeyValuePair<TKey, TValue>[], arrayIndex: number): void;
    /**
     * Performs specified actions for each item in the LinkedList.
     *
     * @param {Delegate} delegate
     * The function with specified actions for each item.
     *
     * @param {any} thisArg
     * Sets the this-keyword in each function call.
     */
    forEach<TReturn>(delegate: (item: KeyValuePair<TKey, TValue>, enumerable: LinkedList<TKey, TValue>) => (TReturn | void), thisArg?: any): (TReturn | undefined);
    /**
     * Returns an enumerator that moves through the LinkedList.
     *
     * @return {IDictionaryEnumerator}
     * Returns the enumerator for the LinkedList.
     */
    getEnumerator(): IDictionaryEnumerator<TKey, TValue>;
    /**
     * Gets an item value in the LinkedList.
     *
     * @param {TKey} key
     * The key of the item to get.
     *
     * @return {TValue}
     * Returns the value of the item.
     *
     * @exception {ArgumentNullException}
     * The key is null.
     *
     * @exception {KeyNotFoundException}
     * The key is not part of the LinkedList.
     */
    item(key: TKey): TValue;
    /**
     * Returns the type of this object for further reflection.
     *
     * @return {Type}
     * The type of this object.
     */
    getType(): Type;
    /**
     * @ignore
     * Gets an item node in the LinkedList.
     *
     * @param key
     * The key of the item to get.
     *
     * @return {LinkedNode<TKey, TValue> | null}
     * Returns the node of the item.
     */
    private node(key);
    /**
     * Removes an item from the LinkedList.
     *
     * @param {T} item
     * The item to remove from the LinkedList.
     *
     * @return {boolean}
     * Returns true if the item was found and successfully removed from the
     * LinkedList, otherwise false.
     *
     * @exception {ArgumentNullException}
     * The key is null.
     *
     * @exception {NotSupportedException}
     * The LinkedList is read-only.
     */
    remove(item: KeyValuePair<TKey, TValue>): boolean;
    /**
     * Removes the specified item from the LinkedList.
     *
     * @param {TKey} key
     * The key of the item to remove from the LinkedList.
     *
     * @return {boolean}
     * Returns true if the item was found and successfully removed, otherwise
     * false.
     *
     * @exception {ArgumentNullException}
     * The key is null.
     *
     * @exception {NotSupportedException}
     * The LinkedList is read-only.
     */
    removeItem(key: TKey): boolean;
    /**
     * Gets the value of a item with the specified key.
     *
     * @param {TKey} key
     * The key of the item, whose value is wanted.
     *
     * @param {(TValue|undefined)=>any} outValue
     * Return the value of the item, or undefined.
     *
     * @return {boolean}
     * When true is returned, the value of the item has been returned,
     * otherwise false indicates a return of undefined.
     *
     * @exception {ArgumentNullException}
     * The key is null.
     */
    tryGetValue(key: TKey, outValue: (outValue: (TValue | undefined)) => any): boolean;
}
export default LinkedList;
