"use strict";
/*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*!*\

  System Core TypeScript Library

  Copyright (c) TypeScriptLibs and Contributors

  Licensed under the MIT License; you may not use this file except in
  compliance with the License. You may obtain a copy of the MIT License at
  https://typescriptlibs.org/LICENSE.txt

\*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*i*/
Object.defineProperty(exports, "__esModule", { value: true });
const package_1 = require("../../package");
const ArgumentException_1 = require("../Exceptions/ArgumentException");
const ArgumentNullException_1 = require("../Exceptions/ArgumentNullException");
const ArgumentOutOfRangeException_1 = require("../Exceptions/ArgumentOutOfRangeException");
const InvalidOperationException_1 = require("../Exceptions/InvalidOperationException");
const KeyNotFoundException_1 = require("../Exceptions/KeyNotFoundException");
const KeyValuePair_1 = require("./KeyValuePair");
const NotSupportedException_1 = require("../Exceptions/NotSupportedException");
const Object_1 = require("../Object");
const Type_1 = require("../Type");
class LinkedList extends Object_1.default {
    constructor() {
        super();
        this._count = 0;
        this._endNode = null;
        this._startNode = null;
        this._version = new LinkedVersion();
    }
    get count() {
        return this._count;
    }
    get isReadOnly() {
        return false;
    }
    get keys() {
        return LinkedKeyCollection.of(this._startNode, this._count, this._version);
    }
    get values() {
        return LinkedValueCollection.of(this._startNode, this._count, this._version);
    }
    add(item) {
        this.addItem(item.key, item.value);
    }
    addItem(key, value) {
        if (key === undefined
            || key === null) {
            throw new ArgumentNullException_1.default();
        }
        if (this.containsKey(key)) {
            throw new ArgumentException_1.default();
        }
        this._version.counter++;
        let newNode = new LinkedNode(key, value);
        if (this._startNode === null
            || this._endNode === null) {
            this._startNode = newNode;
            this._endNode = newNode;
        }
        else {
            this._endNode.nextNode = newNode;
            newNode.previousNode = this._endNode;
            this._endNode = newNode;
        }
        this._count++;
    }
    clear() {
        this._version.counter++;
        let nextNode = this._startNode;
        while (nextNode !== null) {
            let clearNode = nextNode;
            nextNode = nextNode.nextNode;
            clearNode.dispose();
        }
        this._startNode = null;
        this._endNode = null;
        this._count = 0;
    }
    contains(item) {
        return this.containsKey(item.key);
    }
    containsKey(key) {
        if (key === undefined
            || key === null) {
            throw new ArgumentNullException_1.default();
        }
        return (this.node(key) !== null);
    }
    copyTo(array, arrayIndex) {
        if (arrayIndex < 0) {
            throw new ArgumentOutOfRangeException_1.default();
        }
        if (array.length - arrayIndex < this.count) {
            throw new ArgumentException_1.default();
        }
        let currentNode = this._startNode;
        while (currentNode !== null) {
            array[arrayIndex] = new KeyValuePair_1.default(currentNode.key, currentNode.value);
            currentNode = currentNode.nextNode;
        }
    }
    forEach(delegate, thisArg) {
        let callReturn = undefined;
        let currentNode = this._startNode;
        while (currentNode !== null) {
            callReturn = delegate.call(thisArg, new KeyValuePair_1.default(currentNode.key, currentNode.value), this);
            if (callReturn !== undefined) {
                break;
            }
            currentNode = currentNode.nextNode;
        }
        return callReturn;
    }
    getEnumerator() {
        return LinkedListEnumerator.of(this._startNode, this._version);
    }
    item(key) {
        if (key === undefined
            || key === null) {
            throw new ArgumentNullException_1.default('key');
        }
        let foundNode = this.node(key);
        if (foundNode === null) {
            throw new KeyNotFoundException_1.default();
        }
        return foundNode.value;
    }
    getType() {
        return new Type_1.default(this, package_1.default);
    }
    node(key) {
        let currentStartNode = this._startNode;
        let currentEndNode = this._endNode;
        while (currentStartNode !== null
            && currentEndNode !== null) {
            if (currentStartNode.key.equals(key)) {
                return currentStartNode;
            }
            if (currentEndNode.key.equals(key)) {
                return currentEndNode;
            }
            if (currentStartNode.equals(currentEndNode)
                || currentStartNode.nextNode === null
                || currentStartNode.nextNode.equals(currentEndNode)) {
                return null;
            }
            currentStartNode = currentStartNode.nextNode;
            currentEndNode = currentEndNode.previousNode;
        }
        return null;
    }
    remove(item) {
        return this.removeItem(item.key);
    }
    removeItem(key) {
        if (key === undefined
            || key === null) {
            throw new ArgumentNullException_1.default();
        }
        let foundNode = this.node(key);
        if (foundNode === null) {
            return false;
        }
        this._version.counter++;
        if (foundNode.previousNode === null) {
            if (foundNode.nextNode === null) {
                this._endNode = null;
                this._startNode = null;
            }
            else {
                this._startNode = foundNode.nextNode;
                this._startNode.previousNode = null;
            }
        }
        else {
            if (foundNode.nextNode === null) {
                this._endNode = foundNode.previousNode;
                this._endNode.nextNode = null;
            }
            else {
                foundNode.previousNode.nextNode = foundNode.nextNode;
                foundNode.nextNode.previousNode = foundNode.previousNode;
            }
        }
        this._count--;
        foundNode.dispose();
        return true;
    }
    tryGetValue(key, outValue) {
        if (key === null) {
            throw new ArgumentNullException_1.default();
        }
        let foundNode = this.node(key);
        if (foundNode === null) {
            outValue(undefined);
            return false;
        }
        else {
            outValue(foundNode.value);
            return true;
        }
    }
}
exports.LinkedList = LinkedList;
exports.default = LinkedList;
class LinkedListEnumerator extends Object_1.default {
    constructor(startNode, version) {
        super();
        this._currentNode = null;
        this._currentVersion = version;
        this._startNode = startNode;
        this._startVersionCounter = version.counter;
    }
    static of(startNode, version) {
        return new LinkedListEnumerator(startNode, version);
    }
    get current() {
        if (this._currentNode === undefined
            || this._currentNode === null) {
            return undefined;
        }
        else {
            return new KeyValuePair_1.default(this._currentNode.key, this._currentNode.value);
        }
    }
    get entry() {
        if (this._currentNode === undefined
            || this._currentNode === null) {
            throw new InvalidOperationException_1.default();
        }
        else {
            return new KeyValuePair_1.default(this._currentNode.key, this._currentNode.value);
        }
    }
    get key() {
        if (this._currentNode === undefined
            || this._currentNode === null) {
            throw new InvalidOperationException_1.default();
        }
        else {
            return this._currentNode.key;
        }
    }
    get node() {
        if (this._currentNode === undefined
            || this._currentNode === null) {
            throw new InvalidOperationException_1.default();
        }
        else {
            return this._currentNode;
        }
    }
    get value() {
        if (this._currentNode === undefined
            || this._currentNode === null) {
            throw new InvalidOperationException_1.default();
        }
        else {
            return this._currentNode.value;
        }
    }
    dispose() {
        delete this._currentNode;
        delete this._currentVersion;
        delete this._startNode;
        delete this._startVersionCounter;
    }
    getType() {
        return new Type_1.default(this, package_1.default);
    }
    moveNext() {
        if (this._currentVersion.counter !== this._startVersionCounter) {
            throw new InvalidOperationException_1.default();
        }
        if (this._currentNode === undefined) {
            this._currentNode = this._startNode;
        }
        else if (this._currentNode !== null) {
            this._currentNode = this._currentNode.nextNode;
        }
        return (this._currentNode !== null);
    }
    reset() {
        this._currentNode = undefined;
    }
}
class LinkedKeyCollection extends Object_1.default {
    constructor(startNode, count, version) {
        super();
        this._count = count;
        this._startNode = startNode;
        this._startVersion = version;
    }
    static of(startNode, count, version) {
        return new LinkedKeyCollection(startNode, count, version);
    }
    get count() {
        return this._count;
    }
    get isReadOnly() {
        return true;
    }
    add(item) {
        throw new NotSupportedException_1.default();
    }
    clear() {
        throw new NotSupportedException_1.default();
    }
    contains(item) {
        let found = this.forEach(item => {
            if (item.equals(item)) {
                return true;
            }
        });
        return (found || false);
    }
    copyTo(array, arrayIndex) {
        if (arrayIndex < 0) {
            throw new ArgumentOutOfRangeException_1.default();
        }
        if (array.length - arrayIndex < this.count) {
            throw new ArgumentException_1.default();
        }
        this.forEach(key => array[arrayIndex] = key);
    }
    forEach(delegate, thisArg) {
        let enumerator = this.getEnumerator();
        let callReturn = null;
        while (enumerator.moveNext()) {
            if (enumerator.current === undefined) {
                return;
            }
            callReturn = delegate.call(thisArg, enumerator.current, this);
            if (callReturn !== undefined) {
                break;
            }
        }
        return callReturn;
    }
    getEnumerator() {
        return LinkedKeyCollectionEnumerator.of(this._startNode, this._startVersion);
    }
    getType() {
        return new Type_1.default(this, package_1.default);
    }
    remove(item) {
        throw new NotSupportedException_1.default();
    }
}
class LinkedKeyCollectionEnumerator extends Object_1.default {
    constructor(startNode, version) {
        super();
        this._currentVersion = version;
        this._startNode = startNode;
        this._startVersionCounter = version.counter;
        this.reset();
    }
    static of(startNode, version) {
        return new LinkedKeyCollectionEnumerator(startNode, version);
    }
    get current() {
        if (this._currentNode === undefined
            || this._currentNode === null) {
            return undefined;
        }
        else {
            return this._currentNode.key;
        }
    }
    dispose() {
        delete this._currentNode;
        delete this._currentVersion;
        delete this._startNode;
        delete this._startVersionCounter;
    }
    getType() {
        return new Type_1.default(this, package_1.default);
    }
    moveNext() {
        if (this._currentVersion.counter !== this._startVersionCounter) {
            throw new InvalidOperationException_1.default();
        }
        if (this._currentNode === undefined) {
            this._currentNode = this._startNode;
        }
        else if (this._currentNode !== null) {
            this._currentNode = this._currentNode.nextNode;
        }
        return (this._currentNode !== null);
    }
    reset() {
        this._currentNode = undefined;
    }
}
class LinkedValueCollection extends Object_1.default {
    constructor(startNode, count, version) {
        super();
        this._count = count;
        this._currentVersion = version;
        this._startNode = startNode;
        this._startVersionCounter = version.counter;
    }
    static of(startNode, count, version) {
        return new LinkedValueCollection(startNode, count, version);
    }
    get count() {
        return this._count;
    }
    get isReadOnly() {
        return true;
    }
    add(item) {
        throw new NotSupportedException_1.default();
    }
    clear() {
        throw new NotSupportedException_1.default();
    }
    contains(item) {
        let found = this.forEach(item => {
            if (item.equals(item)) {
                return true;
            }
        });
        return (found || false);
    }
    copyTo(array, arrayIndex) {
        if (arrayIndex < 0) {
            throw new ArgumentOutOfRangeException_1.default();
        }
        if (array.length - arrayIndex < this.count) {
            throw new ArgumentException_1.default();
        }
        this.forEach(value => array[arrayIndex] = value);
    }
    forEach(delegate, thisArg) {
        let enumerator = this.getEnumerator();
        let callReturn = null;
        while (enumerator.moveNext()) {
            if (enumerator.current === undefined) {
                return;
            }
            callReturn = delegate.call(thisArg, enumerator.current, this);
            if (callReturn !== undefined) {
                break;
            }
        }
        return callReturn;
    }
    getEnumerator() {
        return LinkedValueCollectionEnumerator.of(this._startNode, this._currentVersion);
    }
    getType() {
        return new Type_1.default(this, package_1.default);
    }
    remove(item) {
        throw new NotSupportedException_1.default();
    }
}
class LinkedValueCollectionEnumerator extends Object_1.default {
    constructor(startNode, version) {
        super();
        this._currentVersion = version;
        this._startNode = startNode;
        this._startVersionCounter = version.counter;
        this.reset();
    }
    static of(startNode, version) {
        return new LinkedValueCollectionEnumerator(startNode, version);
    }
    get current() {
        if (this._currentNode === undefined
            || this._currentNode === null) {
            return undefined;
        }
        else {
            return this._currentNode.value;
        }
    }
    dispose() {
        delete this._currentNode;
        delete this._currentVersion;
        delete this._startNode;
        delete this._startVersionCounter;
    }
    getType() {
        return new Type_1.default(this, package_1.default);
    }
    moveNext() {
        if (this._currentVersion.counter !== this._startVersionCounter) {
            throw new InvalidOperationException_1.default();
        }
        if (this._currentNode === undefined) {
            this._currentNode = this._startNode;
        }
        else if (this._currentNode !== null) {
            this._currentNode = this._currentNode.nextNode;
        }
        return (this._currentNode !== null);
    }
    reset() {
        this._currentNode = undefined;
    }
}
class LinkedNode extends Object_1.default {
    constructor(key, value) {
        super();
        this.key = key;
        this.nextNode = null;
        this.previousNode = null;
        this.value = value;
    }
    dispose() {
        delete this.key;
        delete this.nextNode;
        delete this.previousNode;
        delete this.value;
    }
    getType() {
        return new Type_1.default(this, package_1.default);
    }
}
class LinkedVersion extends Object_1.default {
    getType() {
        return new Type_1.default(this, package_1.default);
    }
}
